package org.convey;

import java.awt.*;

import java.awt.geom.*;

import java.awt.event.*;

/**
 * @author Andy Ames
 * @version 0.1
 */
public class ControlPointSelectionBuilder extends DefaultSelectionBuilder {

	protected int segmentIndex = -1;

	protected int ctrlPointIndex = -1;

	public ControlPointSelectionBuilder() {
		super();
	}

	public ControlPointSelectionBuilder(GraphicPanel graphicPanel) {
		super(graphicPanel, null);
	}

	public ControlPointSelectionBuilder(GraphicPanel graphicPanel, Graphic selected) {
		super(graphicPanel, selected);
	}

	public void paint(Graphics2D g2d) {
		if(super.selected != null) {

		  // paint control points
		  if(super.selected instanceof ShapeGraphic) {
				ShapeGraphic graphic = (ShapeGraphic)super.selected;
				Rectangle2D rect;
				float[] dash = { 6, 6 };
				int type;
				int[] coords = new int[6];
				int x = 0, y = 0;
				int lastMoveX = 0, lastMoveY = 0;

				g2d.setStroke(new BasicStroke(1.0f));

				for(int i = 0; i < graphic.coordCount(); i++) {
					type = graphic.getCoords(i, coords);
					switch(type) {
						case ShapeGraphic.SEG_CLOSE:
							x = lastMoveX;
							y = lastMoveY;
							break;
						case ShapeGraphic.SEG_CUBICTO:
							rect = new Rectangle2D.Float(coords[0] - 2, coords[1] - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							rect = new Rectangle2D.Float(coords[2] - 2, coords[3] - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							rect = new Rectangle2D.Float(coords[4] - 2, coords[5] - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							rect = new Rectangle2D.Float(x - 2, y - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);

							g2d.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
														BasicStroke.JOIN_BEVEL, 0.0f, dash, 0.0f));
							g2d.drawLine(x, y, coords[0], coords[1]);
							g2d.drawLine(coords[2], coords[3], coords[4], coords[5]);
							break;
						case ShapeGraphic.SEG_LINETO:
							rect = new Rectangle2D.Float(coords[0] - 2, coords[1] - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							rect = new Rectangle2D.Float(x - 2, y - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							break;
						case ShapeGraphic.SEG_MOVETO:
							x = lastMoveX = coords[0];
							y = lastMoveY = coords[1];
							break;
						case ShapeGraphic.SEG_QUADTO:
							rect = new Rectangle2D.Float(coords[0] - 2, coords[1] - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							rect = new Rectangle2D.Float(coords[2] - 2, coords[3] - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);
							rect = new Rectangle2D.Float(x - 2, y - 2, 4, 4);
							g2d.setColor(Color.white);
							g2d.fill(rect);
							g2d.setColor(Color.black);
							g2d.draw(rect);

							g2d.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
														BasicStroke.JOIN_BEVEL, 0.0f, dash, 0.0f));
							g2d.drawLine(x, y, coords[0], coords[1]);
							g2d.drawLine(coords[0], coords[1], coords[2], coords[3]);
							break;
					}
				}
		  } else
				super.paint(g2d);
		}
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e) {
		if(super.selected != null && super.selected instanceof ShapeGraphic) {
			// did we press a control point? Keep it selected. Otherwise, ...
			ShapeGraphic graphic = (ShapeGraphic)super.selected;
			int type;
			int[] coords = new int[6];
			int x = e.getX();
			int y = e.getY();

			for(int i = 0; i < graphic.coordCount(); i++) {
				type = graphic.getCoords(i, coords);
				switch(type) {
					case ShapeGraphic.SEG_CLOSE:
						break;
					case ShapeGraphic.SEG_CUBICTO:
						if(x >= coords[0] - 2 && x <= coords[0] + 2 &&
							 y >= coords[1] - 2 && y <= coords[1] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 0;
						} else if(x >= coords[2] - 2 && x <= coords[2] + 2 &&
							 y >= coords[3] - 2 && y <= coords[3] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 1;
						} else if(x >= coords[4] - 2 && x <= coords[4] + 2 &&
							 y >= coords[5] - 2 && y <= coords[5] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 2;
						}
						break;
					case ShapeGraphic.SEG_LINETO:
						if(x >= coords[0] - 2 && x <= coords[0] + 2 &&
							 y >= coords[1] - 2 && y <= coords[1] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 0;
						}
						break;
					case ShapeGraphic.SEG_MOVETO:
						if(x >= coords[0] - 2 && x <= coords[0] + 2 &&
							 y >= coords[1] - 2 && y <= coords[1] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 0;
						}
						break;
					case ShapeGraphic.SEG_QUADTO:
						if(x >= coords[0] - 2 && x <= coords[0] + 2 &&
							 y >= coords[1] - 2 && y <= coords[1] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 0;
						} else if(x >= coords[2] - 2 && x <= coords[2] + 2 &&
							 y >= coords[3] - 2 && y <= coords[3] + 2) {
							segmentIndex = i;
							ctrlPointIndex = 1;
						}
						break;
				}
				if(segmentIndex != -1)
					break;
			}

			if(segmentIndex == -1)
				super.mousePressed(e);
		} else
			super.mousePressed(e);
	}

	/**
	   * Invoked when a mouse button is pressed on a component and then
	   * dragged.  Mouse drag events will continue to be delivered to
	   * the component where the first originated until the mouse button is
	   * released (regardless of whether the mouse position is within the
	   * bounds of the component).
	 */
	public void mouseDragged(MouseEvent e) {
		if(super.selected != null) {
			if(super.selected instanceof ShapeGraphic) {
				// did we drag a control point? Otherwise, ...
				if(this.segmentIndex != -1) {
					ShapeGraphic graphic = (ShapeGraphic)super.selected;
					int[] coords = new int[6];
					int type = graphic.getCoords(segmentIndex, coords);
					coords[ctrlPointIndex * 2] = e.getX();
					coords[ctrlPointIndex * 2 + 1] = e.getY();
					graphic.setCoords(segmentIndex, type, coords);
					super.graphicPanel.repaint();
				} else
					super.mouseDragged(e);
			} else
				super.mouseDragged(e);
		}
	}

	public void mouseReleased(MouseEvent e) {
		if(this.segmentIndex != -1)
			this.segmentIndex = -1;
	}

}
