package org.convey;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * @author Andy Ames
 * @version 0.1
 */
public class CurveBuilder extends AbstractBuilder {

	protected Vector points = null;

	protected CurveBuilder() {}

	public CurveBuilder(GraphicPanel graphicPanel) {
		super(graphicPanel);
	}

	protected ShapeGraphic createFittedCurve() {
		int N = this.points.size();
		if(N < 4)
			return null;

		ShapeGraphic curve = new ShapeGraphic(super.graphicPanel);

		double[] x_2_N = new double[N];
		double[] y_2_N = new double[N];
		double[] t_N = new double[N];
		double[][] F_N_4 = new double[N][4];
		double[][] FT_4_N = new double[4][N];
		for(int i = 0; i < N; i++) {
			t_N[i] = (double)i / (double)N;

			Point pt = (Point)this.points.elementAt(i);
			x_2_N[i] = (double)pt.x;
			y_2_N[i] = (double)pt.y;

			FT_4_N[0][i] = F_N_4[i][0] = 1.0f;
			FT_4_N[1][i] = F_N_4[i][1] = t_N[i];
			FT_4_N[2][i] = F_N_4[i][2] = Math.pow(t_N[i], 2);
			FT_4_N[3][i] = F_N_4[i][3] = Math.pow(t_N[i], 3);
		}

		double[][] FTxF_4_4 = new double[4][4];
		for(int i = 0; i < 4; i++)
			for(int j = 0; j < 4; j++)
				for(int k = 0; k < N; k++)
					FTxF_4_4[i][j] += FT_4_N[i][k] * F_N_4[k][j];

		double[] FTxx_4 = new double[4];
		double[] FTyy_4 = new double[4];
		for(int i = 0; i < 4; i++) {
			for(int k = 0; k < N; k++) {
				FTxx_4[i] += FT_4_N[i][k] * x_2_N[k];
				FTyy_4[i] += FT_4_N[i][k] * y_2_N[k];
			}
		}

		Matrix Ax = new Matrix(FTxF_4_4);
		Matrix Cx = Ax.lu().solve(new Matrix(FTxx_4, FTxx_4.length));

		Matrix Ay = new Matrix(FTxF_4_4);
		Matrix Cy = Ay.lu().solve(new Matrix(FTyy_4, FTyy_4.length));

		Matrix Q  = new Matrix(4, 2);
		for(int i = 0; i < 4; i++) {
		  Q.set(i,0,Cx.get(i,0));
			Q.set(i,1,Cy.get(i,0));
		}

		double[][] vals = { { 0.0, 0.0, 0.0, 1.0},
											  { 0.0, 0.0, 3.0, -3.0 },
												{ 0.0, 3.0, -6.0, 3.0 },
												{ 1.0, -3.0, 3.0, -1.0 }
											};
		Matrix B = new Matrix(vals);
		Matrix G = B.lu().solve(Q);

		int[] moveto = { (int)G.get(0, 0), (int)G.get(0, 1), 0, 0, 0, 0 };
		curve.addCoords(0, ShapeGraphic.SEG_MOVETO, moveto);

		int[] cubicto = { (int)G.get(1, 0), (int)G.get(1, 1),
										  (int)G.get(2, 0), (int)G.get(2, 1),
										  (int)G.get(3, 0), (int)G.get(3, 1)};
		curve.addCoords(1, ShapeGraphic.SEG_CUBICTO, cubicto);

		return curve;
	}

	public void paint(Graphics2D g2d) {
		if(points != null) {
			Iterator iter = points.iterator();
	  	Point pt;
		  g2d.setStroke(new BasicStroke(1));
			g2d.setColor(Color.black);
	  	while(iter.hasNext()) {
		  	pt = (Point)iter.next();
			  g2d.fillOval(pt.x, pt.y, 1, 1);
			}
		}
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e) {
		points = new Vector();
		points.add(e.getPoint());
		super.graphicPanel.repaint();
	}

	/**
	 * Invoked when a mouse button has been released on a component.
	 */
	public void mouseReleased(MouseEvent e) {
		if(points != null) {

			ShapeGraphic shape = createFittedCurve();
			if(shape != null) {
				super.graphicPanel.addGraphic(shape);
				super.graphicPanel.repaint();
			}
			points = null;
		}
	}

	/**
	* Invoked when a mouse button is pressed on a component and then
	* dragged.  Mouse drag events will continue to be delivered to
	* the component where the first originated until the mouse button is
	* released (regardless of whether the mouse position is within the
	* bounds of the component).
	*/
	public void mouseDragged(MouseEvent e) {
		if(points != null) {
			points.add(e.getPoint());
			super.graphicPanel.repaint();
		}
	}

}
