package org.convey;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.Rectangle2D;
import java.lang.Math.*;

/**
 * @author Ed Braunhut
 * @version 0.1
 */
public class RectangleBuilder extends AbstractBuilder {

	protected Point startPt = null;

	protected Point endPt;

	public RectangleBuilder(GraphicPanel graphicPanel) {
		super(graphicPanel);
	}

	public void paint(Graphics2D g2d){
		if (this.startPt != null) {
			float[] dash = {6,6};
			g2d.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, dash, 0.0f));
			g2d.setColor(Color.black);
			g2d.drawLine(this.startPt.x, this.startPt.y, this.startPt.x, this.endPt.y);
			g2d.drawLine(this.startPt.x, this.endPt.y, this.endPt.x, this.endPt.y);
			g2d.drawLine(this.endPt.x, this.endPt.y, this.endPt.x, this.startPt.y);
			g2d.drawLine(this.endPt.x, this.startPt.y, this.startPt.x, this.startPt.y);
		}
	}
	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e) {
		this.startPt = e.getPoint();
	}

  /**
   * Invoked when a mouse button is pressed on a component and then
   * dragged.  Mouse drag events will continue to be delivered to
   * the component where the first originated until the mouse button is
   * released (regardless of whether the mouse position is within the
   * bounds of the component).
	 */
	public void mouseDragged(MouseEvent e) {
		this.endPt = e.getPoint();
		super.graphicPanel.repaint();


	}
	/**
	 * Invoked when a mouse button has been released on a component.
	 */
	public void mouseReleased(MouseEvent e) {
		this.endPt = e.getPoint();
		if (this.endPt.x - this.startPt.x < 0 &&
			this.endPt.y - this.startPt.y < 0) {
			this.endPt = this.startPt;
			this.startPt = e.getPoint();
		}else if (this.endPt.x - this.startPt.x < 0) {
			this.endPt.x = this.startPt.x;
			this.startPt.x = e.getPoint().x;
		}else if (this.endPt.y - this.startPt.y < 0) {
			this.endPt.y = this.startPt.y;
			this.startPt.y = e.getPoint().y;
		}

		this.finish();
		super.graphicPanel.repaint();

		this.startPt = null;
		this.endPt = null;
	}

	protected void finish() {
		RectangleGraphic rectangle = new RectangleGraphic(super.graphicPanel);
		rectangle.setRectangle(new Rectangle(Math.min(this.startPt.x, this.endPt.x),
																			   Math.min(this.startPt.y, this.endPt.y),
																				 Math.abs(this.endPt.x - this.startPt.x),
																				 Math.abs(this.endPt.y - this.startPt.y)));
		super.graphicPanel.graphics.add(rectangle);
	}

}
