package org.convey;

import java.awt.*;
import java.text.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

/**
 * @author Andy Ames
 * @version 0.1
 */
public class TextGraphic extends RectangularGraphic implements KeyListener {

	protected String text;

	protected TextLayout layout = null;

	protected int cursor = -1;

  public TextGraphic(GraphicPanel graphicPanel) {
		super(graphicPanel);

		this.setText(null);
  }

	/**
	 * Paints the Graphic object onto the Graphics2D object.
	 */
	public void paint(Graphics2D g2d) {
		if(this.text.length() != 0) {
			this.layout = new TextLayout(this.text, g2d.getFont(),
																	 g2d.getFontRenderContext());
			Rectangle2D bounds = layout.getBounds();
			super.rectangle.width = (int)bounds.getWidth();
			super.rectangle.height = (int)bounds.getHeight() + (int)this.layout.getDescent();
			layout.draw(g2d, super.rectangle.x, super.rectangle.y + this.layout.getAscent());

			if(this.cursor != -1) {
				g2d.translate(super.rectangle.x, super.rectangle.y + this.layout.getAscent());

				Shape[] carets = layout.getCaretShapes(this.cursor);
				g2d.draw(carets[0]);
				if(carets[1] != null)
					g2d.draw(carets[1]);

				g2d.translate(-super.rectangle.x, -super.rectangle.y - this.layout.getAscent());
			}
		}

		/*Point pen = new Point(10, 20);

		LineBreakMeasurer measurer = new LineBreakMeasurer(new AttributedCharacterIterator());
		float wrappingWidth = getSize().width - 15;

		while(measurer.getPosition() < fStyledText.length()) {
		  TextLayout layout = measurer.nextLayout(wrappingWidth);

			pen.y += (layout.getAscent());
			float dx = layout.isLeftToRight() ? 0 : (wrappingWidth - layout.getAdvance());

			layout.draw(graphics, pen.x + dx, pen.y);
			pen.y += layout.getDescent() + layout.getLeading();
		}*/
	}

	public void paintSelection(Graphics2D g2d) {
		if(this.cursor == -1)
			super.paintSelection(g2d);
	}

	public boolean contains(Point pt) {
		if(super.contains(pt))
			return true;

		if(super.rectangle.contains(pt))
			return true;

		return false;
	}

	public void setText(String text) {
		if(text == null)
			this.text = "\n";
		else
			this.text = text;
	}

	public String getText() {
		return this.text;
	}

	public void edit(Point pt) {
		Rectangle2D bounds = layout.getBounds();
		TextHitInfo hit = layout.hitTestChar(pt.x - super.rectangle.x,
																			   pt.y - super.rectangle.y - (int)bounds.getY());
		this.cursor = hit.getInsertionIndex();
	}

	public void setCursor(int cursor) {
		if(cursor >= -1 && cursor <= this.text.length())
			this.cursor = cursor;
	}

	public int getCursor() {
		return this.cursor;
	}

	/**
	 * Sets Graphic object's selected state.
	 */
	public void setSelected(boolean selected) {
		super.setSelected(selected);

		if(!selected)
			this.cursor = -1;
	}

	/**
	 * Invoked when a key has been typed. This event occurs when a key press is
	 * followed by a key release.
	 */
	public void keyTyped(KeyEvent e) {
		char ch = e.getKeyChar();
		//if(this.cursor != -1 && ch != KeyEvent.CHAR_UNDEFINED && ch != KeyEvent.VK_BACK_SPACE) {
		if(this.cursor != -1 && !Character.isISOControl(ch)) {
			if(this.text.equals("\n")) {

				// Must do this because cursor may have been placed at position 1. We
				// don't want to increment past the end of the string after swapping the
				// "\n" for the character typed.
				this.cursor = 0;

				this.text = String.valueOf(ch);
			} else if(this.cursor == 0)
				this.text = String.valueOf(ch) + this.text;
			else if(this.cursor == this.text.length())
				this.text = this.text + String.valueOf(ch);
			else
				this.text = this.text.substring(0, this.cursor) +
									  String.valueOf(ch) + this.text.substring(this.cursor);

		  this.cursor++;
		}
	}

	/**
	 * Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e) {
		if(this.cursor != -1) {
			switch(e.getKeyCode()) {
	  		case KeyEvent.VK_LEFT:
		  	  if(cursor > 0)
			  		cursor--;
				  break;
				case KeyEvent.VK_RIGHT:
	  		  if(cursor < this.text.length())
		  			cursor++;
			  	break;
				case KeyEvent.VK_DELETE:
	  			if(cursor < this.text.length()) {
						if(this.text.length() == 1)
							this.text = "\n";
						else if(cursor == 0) {
							this.text = this.text.substring(1);
						} else {
							this.text = this.text.substring(0, cursor) +
												  this.text.substring(cursor + 1);
						}
	  			}
					break;
				case KeyEvent.VK_BACK_SPACE:
	  			if(cursor > 0) {
						if(this.text.length() == 1)
							this.text = "\n";
						else if(cursor == this.text.length()) {
							this.text = this.text.substring(0, this.text.length() - 1);
						} else {
							this.text = this.text.substring(0, cursor - 1) +
												  this.text.substring(cursor);
						}
						cursor--;
	  			}
					break;
				case KeyEvent.VK_END:
					this.cursor = this.text.length();
					break;
				case KeyEvent.VK_HOME:
					this.cursor = 0;
					break;
				case KeyEvent.VK_ESCAPE:
					this.cursor = -1;
					break;
				case KeyEvent.VK_ENTER:
					if(this.cursor == -1)
						this.cursor = this.text.length();
					break;
		  }
		}
	}

	/**
	 * Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) {}

}
