package org.convey;

import javax.swing.event.*;

import java.awt.event.*;

/**
 * @author Ed Braunhut and Andy Ames
 * @version 0.1
 */
public abstract class AbstractBuilder implements Builder {

	protected GraphicPanel graphicPanel;

	public AbstractBuilder() {
		this(null);
	}

	public AbstractBuilder(GraphicPanel graphicPanel) {
		if(graphicPanel == null)
			throw new NullPointerException();

		this.graphicPanel = graphicPanel;
	}

	public GraphicPanel getGraphicPanel() {
		return this.graphicPanel;
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e) {
		this.graphicPanel.requestFocus();
	}

  	/**
   	 * Invoked when a mouse button has been released on a component.
	 */
	public void mouseReleased(MouseEvent e) {}

	/**
	 * Invoked when the mouse has been clicked on a component.
	 */
	public void mouseClicked(MouseEvent e) {}

	/**
	* Invoked when a mouse button is pressed on a component and then
	* dragged.  Mouse drag events will continue to be delivered to
	* the component where the first originated until the mouse button is
	* released (regardless of whether the mouse position is within the
	* bounds of the component).
	*/
	public void mouseDragged(MouseEvent e) {}

	/**
	 * Invoked when the mouse button has been moved on a component
	 * (with no buttons or mouse buttons pressed)
	 */
	public void mouseMoved(MouseEvent e) {}

    /**
	 * Invoked when the mouse exits a component.
	 */
	public void mouseExited(MouseEvent e) {}

    /**
     * Invoked when the mouse enters a component.
	 */
	public void mouseEntered(MouseEvent e) {}

	/**
	 * Invoked when a key has been typed. This event occurs when a key press is
	 * followed by a key release.
	 */
	public void keyTyped(KeyEvent e) {
	  int i = 5;
	}

	/**
	 * Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e) {
		int i = 5;
	}

	/**
	 * Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) {
		int i = 5;
	}

}
