package org.convey;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * CubicBuilder.java
 *
 * @author Ed Braunhut
 * @version 0.1
 */
public class CubicBuilder extends QuadBuilder {
	protected int[] cubicTo = {0, 0, 0, 0, 0, 0};

	public CubicBuilder() {}

	public CubicBuilder(GraphicPanel graphicPanel){
		super(graphicPanel);
	}

	public void paint(Graphics2D g2d) {

		if (this.shape != null) {
			float[] dash = {6,6};
			g2d.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, dash, 0.0f));
			g2d.setColor(Color.black);
			g2d.drawLine(this.currPt.x, this.currPt.y, this.secondPt.x, this.secondPt.y);
			g2d.drawLine(this.currPt.x, this.currPt.y, this.startPt.x, this.startPt.y);
		}
	}
	/**
	 * Invoked while extending the Shape.
	 * Note that the various Pts just indicate what part of the building process
	 * is currently happening and pass on values to ShapeGraphic methods for
	 * actually creating the shape.
	 */
	public void mouseClicked(MouseEvent e) {
		super.mouseClicked(e);
	}

	public void doClick(Point currPt) {
		if (shape == null) {//if build is terminated here it is a point
			shape = new ShapeGraphic(super.getGraphicPanel());
			int[] cubicTo = {this.startPt.x, this.startPt.y, 0, 0, 0, 0};
			shape.addCoords(0, ShapeGraphic.SEG_MOVETO,cubicTo);
			super.graphicPanel.addGraphic(shape);
		}else if (clickCount == 2) {
			//begin creating default quad curve (line) with midpt as the ctrlPt
			cubicTo[0] = this.ctrlPt1.x;
			cubicTo[1] = this.ctrlPt1.y;
			cubicTo[2] = this.ctrlPt1.x;
			cubicTo[3] = this.ctrlPt1.y;
			cubicTo[4] = this.secondPt.x;
			cubicTo[5] = this.secondPt.y;
			shape.addCoords(1,ShapeGraphic.SEG_CUBICTO, cubicTo);
		}else if (clickCount == 3) {
			cubicTo[0] = this.ctrlPt1.x;
			cubicTo[1] = this.ctrlPt1.y;
			cubicTo[4] = this.secondPt.x;
			cubicTo[5] = this.secondPt.y;
			shape.setCoords(1,ShapeGraphic.SEG_CUBICTO, cubicTo);
		}else if (clickCount == 4) {
			this.finishShape(ctrlPt2);
		}
		super.graphicPanel.repaint();

	}
	public void finishShape(Point ctrlPt) {
		cubicTo[2] = ctrlPt.x;
		cubicTo[3] = ctrlPt.y;
		shape.setCoords(1, ShapeGraphic.SEG_CUBICTO, cubicTo);  //use clicked ctrlPt instead of midPoint default
		super.graphicPanel.repaint();
		clickCount = 0;
		shape = null;
	}
	public void mouseMoved(MouseEvent e) {
		super.mouseMoved(e);
		if (shape != null) {
			if (clickCount == 2)
				shape.setCoords(1, ShapeGraphic.SEG_CUBICTO,
								new int[] {e.getPoint().x, e.getPoint().y,
										   e.getPoint().x, e.getPoint().y,cubicTo[4],cubicTo[5]});
			else if (clickCount == 3)
				shape.setCoords(1, ShapeGraphic.SEG_CUBICTO,
							    new int[] {cubicTo[0],cubicTo[1],
										   e.getPoint().x, e.getPoint().y, cubicTo[4], cubicTo[5]});
			super.graphicPanel.repaint();
		}
	}

}

