package org.convey;

import java.util.*;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * @author Andy Ames
 * @version 0.1
 */
public class DefaultSelectionBuilder extends AbstractSelectionBuilder {

	protected Point lastPt = null;

	public DefaultSelectionBuilder() {
		super();
	}

	public DefaultSelectionBuilder(GraphicPanel graphicPanel) {
		super(graphicPanel, null);
  }

  	public DefaultSelectionBuilder(GraphicPanel graphicPanel, Graphic selected) {
		super(graphicPanel, selected);
  }

	/**
	 * Invoked when the mouse has been clicked on a component.
	 */
	public void mouseClicked(MouseEvent e) {
		if(super.selected != null && super.selected instanceof TextGraphic && e.getClickCount() == 2) {
			((TextGraphic)super.selected).edit(e.getPoint());
			super.graphicPanel.repaint();
		}
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e) {
		super.graphicPanel.requestFocus();

		ListIterator iter = graphicPanel.graphicListIterator(this.graphicPanel.graphicCount());
		while(iter.hasPrevious()) {
			Graphic graphic = (Graphic)iter.previous();
			if(graphic.contains(e.getPoint())) {
				if(super.selected != graphic) {
					if(super.selected != null) {
						super.selected.setSelected(false);
						graphicPanel.repaint();
					}
					super.selected = graphic;
					super.graphicPanel.bringToFront(super.selected);
					graphic.setSelected(true);
					graphicPanel.repaint();
				}
				this.lastPt = e.getPoint();
				return;
			}
		}
		if(super.selected != null) {
			super.selected.setSelected(false);
			super.graphicPanel.repaint();
			super.selected = null;
			this.lastPt = null;
		}
	}

	/**
   * Invoked when a mouse button is pressed on a component and then
   * dragged.  Mouse drag events will continue to be delivered to
   * the component where the first originated until the mouse button is
   * released (regardless of whether the mouse position is within the
   * bounds of the component).
	 */
	public void mouseDragged(MouseEvent e) {
		if(super.selected != null) {
			//super.graphicPanel.repaint(super.selected.bounds());
				Point pt = e.getPoint();
		  	super.selected.translate(pt.x - lastPt.x, pt.y - lastPt.y);
		  	//super.graphicPanel.repaint(super.selected.bounds());
			super.graphicPanel.repaint();
			lastPt = e.getPoint();
		}
	}

	/**
	 * Invoked when a key has been typed. This event occurs when a key press is
	 * followed by a key release.
	 */
	public void keyTyped(KeyEvent e) {
		if(super.selected != null && super.selected instanceof TextGraphic) {
			TextGraphic textGraphic = (TextGraphic)super.selected;
			if(textGraphic.getCursor() != -1) {
				textGraphic.keyTyped(e);
	  		super.graphicPanel.repaint();
			}
		}
	}

	/**
	 * Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e) {
		if(super.selected != null && super.selected instanceof TextGraphic) {
			TextGraphic textGraphic = (TextGraphic)super.selected;
			if(textGraphic.getCursor() != -1) {
				textGraphic.keyPressed(e);
	  		super.graphicPanel.repaint();
			}
		}
	}

	/**
	 * Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) {
		if(super.selected != null && super.selected instanceof TextGraphic) {
			TextGraphic textGraphic = (TextGraphic)super.selected;
			if(textGraphic.getCursor() != -1) {
				textGraphic.keyReleased(e);
	  		super.graphicPanel.repaint();
			}
		}
	}

}
