package org.convey;

import java.awt.*;

import java.awt.geom.*;

/**
 * Graphic interface implemented by all Graphic objects.
 *
 * @author Andy Ames
 * @version 0.1
 */
public abstract class Graphic {

	/**
	 * Containing GraphicPanel instance.
	 */
	protected GraphicPanel graphicPanel;

	/**
   * Flag indicating if the Graphic object is selected.
   */
	protected boolean selected = false;

	/**
	 * Constructor taking containing GraphicPanel instance.
	 */
	public Graphic(GraphicPanel graphicPanel) {
		this.graphicPanel = graphicPanel;
	}

	/**
	 * Paints the Graphic object onto the Graphics2D object.
	 */
	public abstract void paint(Graphics2D g2d);

	public abstract void paintSelection(Graphics2D g2d);

	public abstract boolean contains(Point pt);

	public abstract void translate(int tx, int ty);

	/**
	 * Determines if Graphic is selected.
	 */
	public boolean isSelected() {
		return this.selected;
	}

	/**
	 * Sets Graphic object's selected state.
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	public GraphicPanel getGraphicPanel() {
		return this.graphicPanel;
	}

}
