package org.convey;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;

/**
 * Main JFrame for the Graphic Editor. A BorderLayout layout manager is used
 * to manage a GraphicPanel in the BorderLayout.CENTER position and a JToolBar
 * in the BorderLayout.NORTH position.
 *
 * The JToolBar has buttons for creating and manipulating the Graphic objects in
 * the GraphicPanel.
 *
 * @author Andy Ames
 * @version 0.1
 */
public class GraphicFrame extends JFrame {

	protected GraphicPanel graphicPanel;

	protected JToolBar toolBar;

	protected Action selectAction;

	protected JToggleButton selectButton;

	protected Action selectCompAction;

	protected JToggleButton selectCompButton;

	protected Action addPointAction;

	protected JToggleButton addPointButton;

	protected Action removePointAction;

	protected JToggleButton removePointButton;

	protected Action createTextAction;

	protected JToggleButton createTextButton;

	protected Action createRectangleAction;

	protected JToggleButton createRectangleButton;

	protected Action createOvalAction;

	protected JToggleButton createOvalButton;

	protected Action createPolylineAction;

	protected JToggleButton createPolylineButton;

	protected Action createPolygonAction;

	protected JToggleButton createPolygonButton;

	protected Action createQuadcurveAction;

	protected JToggleButton createQuadcurveButton;

	protected Action createCubiccurveAction;

	protected JToggleButton createCubiccurveButton;

	protected Action createCurveAction;

	protected JToggleButton createCurveButton;

	protected ButtonGroup group;

	/**
	 * Default constructor. Here, GraphicFrame sets up its components and shows
	 * itself.
	 */
	public GraphicFrame() {
		super("Convey 0.1");
		super.setSize(800, 700);

		WindowListener l = new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				System.exit(0);
			}
		};
		super.addWindowListener(l);

		super.getContentPane().setLayout(new BorderLayout(5, 5));

		this.group = new ButtonGroup();
		this.toolBar = new JToolBar(JToolBar.HORIZONTAL);

		this.selectAction = new AbstractAction("Select") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new DefaultSelectionBuilder(graphicPanel));
			}
		};

		this.selectButton = new JToggleButton(this.selectAction);
		this.selectButton.setSelected(true);
		this.selectButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.selectButton);
		this.toolBar.add(this.selectButton);

		this.selectCompAction = new AbstractAction("Modify") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new ControlPointSelectionBuilder(graphicPanel));
			}
		};
		this.selectCompButton = new JToggleButton(this.selectCompAction);
		this.selectCompButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.selectCompButton);
		this.toolBar.add(this.selectCompButton);

		this.addPointAction = new AbstractAction("+Pnt") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new CtrlPointAddBuilder(graphicPanel));
			}
		};
		this.addPointButton = new JToggleButton(this.addPointAction);
		this.addPointButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.addPointButton);
		this.toolBar.add(this.addPointButton);

		this.removePointAction = new AbstractAction("-Pnt") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new CtrlPointRemoveBuilder(graphicPanel));
			}
		};
		this.removePointButton = new JToggleButton(this.removePointAction);
		this.removePointButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.removePointButton);
		this.toolBar.add(this.removePointButton);

		this.createTextAction = new AbstractAction("Text") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new TextBuilder(graphicPanel));
			}
		};
		this.createTextButton = new JToggleButton(this.createTextAction);
		this.createTextButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createTextButton);
		this.toolBar.add(this.createTextButton);

		this.createRectangleAction = new AbstractAction("Rectangle") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new RectangleBuilder(graphicPanel));
			}
		};
		this.createRectangleButton = new JToggleButton(this.createRectangleAction);
		this.createRectangleButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createRectangleButton);
		this.toolBar.add(this.createRectangleButton);

		this.createOvalAction = new AbstractAction("Ellipse") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new EllipseBuilder(graphicPanel));
			}
		};
		this.createOvalButton = new JToggleButton(this.createOvalAction);
		this.createOvalButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createOvalButton);
		this.toolBar.add(this.createOvalButton);

		this.createPolylineAction = new AbstractAction("Polyline") {
			public void actionPerformed(ActionEvent evt){
				graphicPanel.setBuilder(new PolylineBuilder(graphicPanel));
			}
		};
		this.createPolylineButton = new JToggleButton(this.createPolylineAction);
		this.createPolylineButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createPolylineButton);
		this.toolBar.add(this.createPolylineButton);

		//Button to invoke PolygonBuilder
		this.createPolygonAction = new AbstractAction("Polygon") {
			public void actionPerformed(ActionEvent evt){
				graphicPanel.setBuilder(new PolygonBuilder(graphicPanel));
			}
		};
		this.createPolygonButton = new JToggleButton(this.createPolygonAction);
		this.createPolygonButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createPolygonButton);
		this.toolBar.add(this.createPolygonButton);

		//Button to invoke QuadcurveBuilder
		this.createQuadcurveAction = new AbstractAction("Qcurve") {
			public void actionPerformed(ActionEvent evt){
				graphicPanel.setBuilder(new QuadBuilder(graphicPanel));
			}
		};
		this.createQuadcurveButton = new JToggleButton(this.createQuadcurveAction);
		this.createQuadcurveButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createQuadcurveButton);
		this.toolBar.add(this.createQuadcurveButton);

		this.createCubiccurveAction = new AbstractAction("CCurve") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new CubicBuilder(graphicPanel));
			}
		};
		this.createCubiccurveButton = new JToggleButton(this.createCubiccurveAction);
		this.createCubiccurveButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createCubiccurveButton);
		this.toolBar.add(this.createCubiccurveButton);

		this.createCurveAction = new AbstractAction("Free Curve") {
			public void actionPerformed(ActionEvent evt) {
				graphicPanel.setBuilder(new CurveBuilder(graphicPanel));
			}
		};
		this.createCurveButton = new JToggleButton(this.createCurveAction);
		this.createCurveButton.setMargin(new Insets(1, 1, 1, 1));
		this.group.add(this.createCurveButton);
		this.toolBar.add(this.createCurveButton);

		super.getContentPane().add(this.toolBar, BorderLayout.NORTH);

		this.graphicPanel = new GraphicPanel(this);
		super.getContentPane().add(this.graphicPanel, BorderLayout.CENTER);

		this.graphicPanel.setBuilder(new DefaultSelectionBuilder(this.graphicPanel));

		super.show();

	}

	/**
	 * Main method invoked when Graphic Editor application starts. All we do is
	 * construct an instance of GraphicFrame. Its constructor will set up its
	 * components and show itself.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			e.printStackTrace();
		}

		new GraphicFrame();
	}

}
