package org.convey;

import java.awt.*;
import java.awt.event.*;
//import javax.swing.*;
import java.awt.geom.*;
import java.util.ListIterator;

/**
 * @author Ed Braunhut
 * @version 0.1
 */
public class PolylineBuilder extends AbstractBuilder {

	protected int clickCount = 0;

	protected ShapeGraphic shape = null;

	protected ShapeGraphic rubberBand = null;

	protected int[] guideValues = {0, 0, 0, 0, 0, 0};

	protected Point currPt = null;

	protected Point prevPt = null;

	protected Point ctrlPt1 = null;

	protected Point ctrlPt2 = null;

	protected Point startPt = null;

	protected Point secondPt = null;

	protected PolylineBuilder() {}

	public PolylineBuilder(GraphicPanel graphicPanel) {
		super(graphicPanel);
	}
	/**
	 *	paint the rectangle bounding current and previous rubberband line
	 *	this will also call the other ShapeGraphics repaint method to
	 *  repaint the portions contained in bounding rectangle
	 *
	 */
	public void paint(Graphics2D g2d) {

		if (this.shape != null) {
			float[] dash = {6,6};
			g2d.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, dash, 0.0f));
			g2d.setColor(Color.black);

			g2d.drawLine(this.currPt.x, this.currPt.y, this.prevPt.x, this.prevPt.y);
		}
	}

	/**
   	* Invoked when the mouse button has been moved on a component
   	* (with no buttons or mouse buttons pressed)
 	*/
	public void mouseMoved(MouseEvent e) {
		if (shape != null) {
			currPt = e.getPoint();
			super.graphicPanel.repaint();
		}
	}

  	/**
   	 * Invoked when the mouse has been clicked on a component.
	 */
	public void mouseClicked(MouseEvent e) {
		if (!(e.isControlDown()) &&
			!(e.isAltDown()) &&
		    !(e.isMetaDown()) &&
		    !(e.isShiftDown() )) { /*no stopping key detected*/

			if (this.clickCount == 0) {
				this.currPt = e.getPoint();
				startPt = e.getPoint();
				secondPt = e.getPoint();
				prevPt = e.getPoint();
				ctrlPt1 = e.getPoint();
				ctrlPt2 = e.getPoint();
			}
			updatePts(this.currPt);
			doClick(currPt);

		} else if (e.isControlDown() ||
				   e.isAltDown() ||
		    	   e.isMetaDown() ||
		    	   e.isShiftDown() ) { /*stopping key detected*/
			finishShape(this.currPt);
			super.graphicPanel.repaint();
		}
	}
	/**
	 * Invoked when the mouse exits a component.
	 */
	public void mouseExited(MouseEvent e) {
		this.currPt = this.prevPt;
		super.graphicPanel.repaint();
	}

	/**
	 * Invoked when completing the Shape
	 */
	public void finishShape(Point currPt) {
		int[] coords = {currPt.x, currPt.y, 0, 0, 0, 0};
		shape.addCoords(shape.coordCount(),ShapeGraphic.SEG_LINETO, coords);
		this.clickCount = 0;
		shape = null;
	}
	/**
	 * Invoked while extending the Shape
	 */
	public void doClick(Point currPt) {
		int[] coords = {this.currPt.x, this.currPt.y, 0, 0, 0, 0};
		if (shape == null) {
			shape = new ShapeGraphic(super.getGraphicPanel());
			shape.addCoords(0, ShapeGraphic.SEG_MOVETO,coords);
			super.graphicPanel.addGraphic(shape);
		}else {
			shape.addCoords(shape.coordCount(),ShapeGraphic.SEG_LINETO, coords);
		}
		super.graphicPanel.repaint();
	}
	public void updatePts(Point currPt) {
		this.clickCount = (this.clickCount)%5 + 1;
		switch (this.clickCount) {
			case 1:
				break;
			case 2:
				this.secondPt = this.currPt;
				this.ctrlPt1.x = (int)((this.secondPt.x + this.startPt.x)/2);
				this.ctrlPt1.y = (int)((this.secondPt.y + this.startPt.y)/2);
				this.ctrlPt2.x = (int)((this.secondPt.x + this.startPt.x)/2);
				this.ctrlPt2.y = (int)((this.secondPt.y + this.startPt.y)/2);
				break;
			case 3:
				this.ctrlPt1 = this.currPt;
				this.ctrlPt2 = this.currPt;
				break;
			case 4:
				this.ctrlPt2 = this.currPt;
		}
		this.prevPt = this.currPt;
	}
}
