/**
 * QuadBuilder.java
 *
 * @author Ed Braunhut
 * @version 0.1
 */

package org.convey;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;



public class QuadBuilder extends PolygonBuilder {

	protected int[] quadTo = {0, 0, 0, 0, 0, 0};

	public QuadBuilder() {}

	public QuadBuilder(GraphicPanel graphicPanel){
		super(graphicPanel);
	}
	/**
	 * Invoked while extending the Shape.
	 * Note that the various Pts just indicate what part of the building process
	 * is currently happening and pass on values to ShapeGraphic methods for
	 * actually creating the shape.
	 */
	public void mouseClicked(MouseEvent e) {
		super.mouseClicked(e);
	}

	public void doClick(Point currPt) {
		if (shape == null) {//if build is terminated here it is a point
			shape = new ShapeGraphic(super.getGraphicPanel());
			int[] quadTo = {this.startPt.x, this.startPt.y, 0, 0, 0, 0};
			shape.addCoords(0, ShapeGraphic.SEG_MOVETO,quadTo);
			super.graphicPanel.addGraphic(shape);
		}else if (clickCount == 2) {
			//begin creating default quad curve (line) with midpt as the ctrlPt
			quadTo[0] = this.ctrlPt1.x;
			quadTo[1] = this.ctrlPt1.y;
			quadTo[2] = this.secondPt.x;
			quadTo[3] = this.secondPt.y;
			shape.addCoords(1,ShapeGraphic.SEG_QUADTO, quadTo);
		}else if (clickCount == 3) {
			this.finishShape(ctrlPt1);
		}
		super.graphicPanel.repaint();

	}
	public void finishShape(Point ctrlPt) {
		quadTo[0] = ctrlPt.x;
		quadTo[1] = ctrlPt.y;
		shape.setCoords(1, ShapeGraphic.SEG_QUADTO, quadTo);  //use clicked ctrlPt instead of midPoint default
		super.graphicPanel.repaint();
		clickCount = 0;
		shape = null;
	}
	public void mouseMoved(MouseEvent e) {
		super.mouseMoved(e);
		if (shape != null && clickCount == 2) {
			shape.setCoords(1, ShapeGraphic.SEG_QUADTO,
							new int[] {e.getPoint().x, e.getPoint().y, quadTo[2],quadTo[3],0,0});
			super.graphicPanel.repaint();
		}
	}

}

