package org.convey;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

/**
 * @author Andy Ames
 * @version 0.1
 */
public class TextBuilder extends AbstractSelectionBuilder {

	public TextBuilder(GraphicPanel graphicPanel) {
		super(graphicPanel);
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e) {
		super.mousePressed(e);

		if(super.selected != null) {
			TextGraphic textGraphic = (TextGraphic)super.selected;

			if(textGraphic.contains(e.getPoint())) {
				textGraphic.edit(e.getPoint());
				super.graphicPanel.repaint();
				return;
			} else
				textGraphic.setSelected(false);
		}

		ListIterator iter = graphicPanel.graphicListIterator(graphicPanel.graphicCount());
		while(iter.hasPrevious()) {
			Graphic graphic = (Graphic)iter.previous();
			if(graphic instanceof TextGraphic && graphic.contains(e.getPoint())) {
				if(super.selected != graphic) {
					if(super.selected != null) {
						super.selected.setSelected(false);
						graphicPanel.repaint();
					}
					super.selected = graphic;
					super.graphicPanel.bringToFront(super.selected);
					graphic.setSelected(true);
					((TextGraphic)super.selected).edit(e.getPoint());
					graphicPanel.repaint();
				}
				return;
			}
		}

		TextGraphic textGraphic = new TextGraphic(super.graphicPanel);

		Point pt = e.getPoint();

		textGraphic.setRectangle(new Rectangle(pt));
		textGraphic.setCursor(0);

		super.graphicPanel.addGraphic(textGraphic);

		super.setSelected(textGraphic);

		super.graphicPanel.repaint();
	}

	/**
	 * Invoked when a key has been typed. This event occurs when a key press is
	 * followed by a key release.
	 */
	public void keyTyped(KeyEvent e) {
		if(super.selected != null) {
			TextGraphic textGraphic = (TextGraphic)super.selected;

			textGraphic.keyTyped(e);
  		super.graphicPanel.repaint();
		}
	}

	/**
	 * Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e) {
		if(super.selected != null) {
			TextGraphic textGraphic = (TextGraphic)super.selected;

			textGraphic.keyPressed(e);
  		super.graphicPanel.repaint();
		}
	}

	/**
	 * Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) {
		if(super.selected != null) {
			TextGraphic textGraphic = (TextGraphic)super.selected;

			textGraphic.keyReleased(e);
  		super.graphicPanel.repaint();
		}
	}

}
